@@ -1,6 +1,6 @@
 /* Plzip - Massively parallel implementation of lzip
    Copyright (C) 2009 Laszlo Ersek.
-   Copyright (C) 2009-2022 Antonio Diaz Diaz.
+   Copyright (C) 2009-2024 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -266,22 +266,21 @@ done:
 
 // start the workers and wait for them to finish.
 int decompress( const unsigned long long cfile_size, int num_workers,
-                const int infd, const int outfd, const Pretty_print & pp,
-                const int debug_level, const int in_slots,
-                const int out_slots, const bool ignore_trailing,
-                const bool loose_trailing, const bool infd_isreg,
-                const bool one_to_one )
+                const int infd, const int outfd, const Cl_options & cl_opts,
+                const Pretty_print & pp, const int debug_level,
+                const int in_slots, const int out_slots,
+                const bool infd_isreg, const bool one_to_one )
   {
   if( !infd_isreg )
-    return dec_stream( cfile_size, num_workers, infd, outfd, pp, debug_level,
-                       in_slots, out_slots, ignore_trailing, loose_trailing );
+    return dec_stream( cfile_size, num_workers, infd, outfd, cl_opts, pp,
+                       debug_level, in_slots, out_slots );
 
-  const Lzip_index lzip_index( infd, ignore_trailing, loose_trailing );
+  const Lzip_index lzip_index( infd, cl_opts );
   if( lzip_index.retval() == 1 )	// decompress as stream if seek fails
     {
     lseek( infd, 0, SEEK_SET );
-    return dec_stream( cfile_size, num_workers, infd, outfd, pp, debug_level,
-                       in_slots, out_slots, ignore_trailing, loose_trailing );
+    return dec_stream( cfile_size, num_workers, infd, outfd, cl_opts, pp,
+                       debug_level, in_slots, out_slots );
     }
   if( lzip_index.retval() != 0 )	// corrupt or invalid input file
     {
